EXPORTS
	Init_glib2
	mGLib		DATA
	rbgobj_cType	DATA
	rbgobj_id_children DATA
	rbgobj_add_relative
	rbgobj_invalidate_relatives
	rbgobj_add_relative_removable
	rbgobj_get_relative_removable
	rbgobj_remove_relative
	rbgobj_remove_relative_all
	rbgobj_define_class
	rbgobj_define_class_dynamic
	rbgobj_register_mark_func
	rbgobj_register_free_func
	rbgobj_instance_from_ruby_object
	rbgobj_ruby_object_from_instance
	rbgobj_ruby_object_from_instance2
	rbgobj_ruby_object_from_instance_with_unref
	rbgobj_instance_unref
	rbgobj_get_ruby_object_from_gobject
	rbgobj_create_object
	rbgobj_gobject_new
	rbgobj_gobject_initialize
	rbgobj_boxed_alloc_func
	rbgobj_boxed_create
	rbgobj_boxed_get
	rbgobj_boxed_get_default
	rbgobj_boxed_not_copy_obj
	rbgobj_boxed_unown
	rbgobj_make_boxed
	rbgobj_make_boxed_raw
	rbgobj_make_boxed_default
	rbgobj_add_abstract_but_create_instance_class
	rbgobj_gtype_get
	rbgobj_gtype_new
	rbgobj_gvalue_to_rvalue
	rbgobj_gvalue_to_rvalue_unset
	rbgobj_rvalue_to_gvalue
	rbgobj_initialize_gvalue
	rbgobj_initialize_object
	rbgobj_class_info_lookup
	rbgobj_class_info_lookup_by_gtype
	rbgobj_class_info_define
	rbgobj_lookup_class
	rbgobj_lookup_class_by_gtype
	rbgobj_gtype_to_ruby_class
	rbgobj_gtype_from_ruby
	rbgobj_ptr2cptr
	rbgobj_ptr_new
	rbgobj_make_enum
	rbgobj_get_enum
	rbgobj_make_flags
	rbgobj_get_flags
	rbgobj_enum_alloc_func
	rbgobj_flags_alloc_func
	rbgobj_register_g2r_func
	rbgobj_register_r2g_func
	rbgobj_register_class
	rbgobj_register_property_getter
	rbgobj_register_property_setter
	rbgobj_class_init_func
	rbgobj_register_type
	rbgobj_object_alloc_func
	rbgobj_object_add_relative
	rbgobj_object_remove_relative
	rbgobj_object_remove_relatives
	rbgobj_set_signal_func
	rbgobj_get_signal_func
	rbgobj_set_signal_call_func
	rbgobj_get_signal_call_func
	rbgobj_add_constants
	rbgobj_constant_remap
	rbgobj_signal_new
	rbgobj_signal_wrap
	g_rclosure_new
	g_rclosure_new_call
	g_rclosure_attach
	g_rclosure_attach_gobject
	g_rclosure_detach
	g_rclosure_detach_gobject
	g_rclosure_set_tag
	rbgobj_ruby_value_get_type
	g_value_get_ruby_value
	g_value_set_ruby_value
	g_key_file_get_type
	rbg_mGLib
	rbg_cGLibObject
	rbg_rval_inspect
	rbg_string_value_ptr
	rbg_rval2cstr
	rbg_rval2cstr_raw
	rbg_rval2cstr_ptr
	rbg_rval2cstr_accept_nil
	rbg_rval2cstr_raw_accept_nil
	rbg_rval2cstr_ptr_accept_nil
	rbg_rval2cstr_accept_symbol
	rbg_rval2cstr_accept_symbol_accept_nil
	rbg_rval2glibid
	rbg_cstr2rval
        rbg_cstr2rval_len
        rbg_cstr2rval_len_free
        rbg_cstr2rval_with_encoding
        rbg_cstr2rval_len_with_encoding
	rbg_cstr2rval_free
	rbg_cstr2rval_with_free
        rbg_filename_to_ruby
        rbg_filename_to_ruby_free
        rbg_filename_from_ruby
        rbg_filename_gslist_to_array_free
        rbg_rval2strv
        rbg_rval2strv_accept_nil
        rbg_rval2strv_dup
        rbg_rval2strv_dup_accept_nil
        rbg_rval2filenamev
        rbg_rval2filenamev_accept_nil
        rbg_strv2rval
        rbg_strv2rval_free
        rbg_rval2gbooleans
        rbg_rval2gints
        rbg_rval2gint8s
        rbg_rval2guint8s
        rbg_rval2guint16s
        rbg_rval2guint32s
        rbg_rval2gdoubles
        rbg_gints2rval
        rbg_gints2rval_free
        rbg_glist2rval
        rbg_gslist2rval
        rbg_glist2rval_with_type
        rbg_gslist2rval_with_type
	rbg_rval2glist
	rbg_rval2gslist
	rbg_define_method
	rbg_define_private_method
	rbg_define_singleton_method
	rbg_to_array
	rbg_to_hash
	rbg_check_array_type
	rbg_check_hash_type
	rbg_scan_options
	rbg_inspect
	rbg_interrupt_source_new
	rbg_name_to_nick
	rbg_memzero
	rbgutil_id_module_eval		DATA
	rbgutil_def_setters
	rbgutil_glist2ary
	rbgutil_glist2ary_boxed
	rbgutil_glist2ary_string
	rbgutil_glist2ary_and_free
	rbgutil_glist2ary_boxed_and_free
	rbgutil_glist2ary_string_and_free
	rbgutil_gslist2ary
	rbgutil_gslist2ary_boxed
	rbgutil_gslist2ary_and_free
	rbgutil_gslist2ary_boxed_and_free
	rbgutil_set_properties
	rbgutil_glibid_r2g_func
	rbgutil_sym_g2r_func
	rbgutil_protect
	rbgutil_on_callback_error
	rbgutil_invoke_callback
	rbgutil_start_callback_dispatch_thread
	rbgutil_stop_callback_dispatch_thread
	rbgutil_string_set_utf8_encoding
	rbgutil_key_equal
	rbgerr_ruby_error_quark
	rbgerr_define_gerror
	rbgerr_gerror2exception
	rbgobj_convert_define
	rbgobj_gc_mark_gvalue
	rbgobj_gc_mark_instance
	rbglib_num_to_uint64
	rbglib_uint64_to_num
	rbglib_num_to_int64
	rbglib_int64_to_num
	rbg_variant_to_ruby
	rbg_variant_from_ruby
	rbg_variant_type_from_ruby

	rbg_gc_guard
	rbg_gc_unguard

	rbg_is_bytes
	rbg_is_object
	rbg_is_value

	g_connect_flags_get_type
	g_signal_flags_get_type
