
/* Generated by glib-mkenums.rb ($Id$) */ 

#ifndef __GLIB_ENUM_TYPES_H__
#define __GLIB_ENUM_TYPES_H__

#include <glib-object.h>

G_BEGIN_DECLS

/* enumerations from "/usr/include/glib-2.0/glib/gbookmarkfile.h" */
GType g_bookmark_file_error_get_type (void);
#define G_TYPE_BOOKMARK_FILE_ERROR (g_bookmark_file_error_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gchecksum.h" */
GType g_checksum_type_get_type (void);
#define G_TYPE_CHECKSUM_TYPE (g_checksum_type_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gconvert.h" */
GType g_convert_error_get_type (void);
#define G_TYPE_CONVERT_ERROR (g_convert_error_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gdate.h" */
GType g_date_dm_y_get_type (void);
#define G_TYPE_DATE_DM_Y (g_date_dm_y_get_type())
GType g_date_weekday_get_type (void);
#define G_TYPE_DATE_WEEKDAY (g_date_weekday_get_type())
GType g_date_month_get_type (void);
#define G_TYPE_DATE_MONTH (g_date_month_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gfileutils.h" */
GType g_file_error_get_type (void);
#define G_TYPE_FILE_ERROR (g_file_error_get_type())
GType g_file_test_get_type (void);
#define G_TYPE_FILE_TEST (g_file_test_get_type())
GType g_file_set_contents_flags_get_type (void);
#define G_TYPE_FILE_SET_CONTENTS_FLAGS (g_file_set_contents_flags_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/ghook.h" */
GType g_hook_flag_mask_get_type (void);
#define G_TYPE_HOOK_FLAG_MASK (g_hook_flag_mask_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gkeyfile.h" */
GType g_key_file_error_get_type (void);
#define G_TYPE_KEY_FILE_ERROR (g_key_file_error_get_type())
GType g_key_file_flags_get_type (void);
#define G_TYPE_KEY_FILE_FLAGS (g_key_file_flags_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gmarkup.h" */
GType g_markup_error_get_type (void);
#define G_TYPE_MARKUP_ERROR (g_markup_error_get_type())
GType g_markup_parse_flags_get_type (void);
#define G_TYPE_MARKUP_PARSE_FLAGS (g_markup_parse_flags_get_type())
GType g_markup_collect_type_get_type (void);
#define G_TYPE_MARKUP_COLLECT_TYPE (g_markup_collect_type_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gmessages.h" */
GType g_log_level_flags_get_type (void);
#define G_TYPE_LOG_LEVEL_FLAGS (g_log_level_flags_get_type())
GType g_log_writer_output_get_type (void);
#define G_TYPE_LOG_WRITER_OUTPUT (g_log_writer_output_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gnode.h" */
GType g_traverse_flags_get_type (void);
#define G_TYPE_TRAVERSE_FLAGS (g_traverse_flags_get_type())
GType g_traverse_type_get_type (void);
#define G_TYPE_TRAVERSE_TYPE (g_traverse_type_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/goption.h" */
GType g_option_flags_get_type (void);
#define G_TYPE_OPTION_FLAGS (g_option_flags_get_type())
GType g_option_arg_get_type (void);
#define G_TYPE_OPTION_ARG (g_option_arg_get_type())
GType g_option_error_get_type (void);
#define G_TYPE_OPTION_ERROR (g_option_error_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gregex.h" */
GType g_regex_error_get_type (void);
#define G_TYPE_REGEX_ERROR (g_regex_error_get_type())
GType g_regex_compile_flags_get_type (void);
#define G_TYPE_REGEX_COMPILE_FLAGS (g_regex_compile_flags_get_type())
GType g_regex_match_flags_get_type (void);
#define G_TYPE_REGEX_MATCH_FLAGS (g_regex_match_flags_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gshell.h" */
GType g_shell_error_get_type (void);
#define G_TYPE_SHELL_ERROR (g_shell_error_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gslice.h" */
GType g_slice_config_get_type (void);
#define G_TYPE_SLICE_CONFIG (g_slice_config_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gspawn.h" */
GType g_spawn_error_get_type (void);
#define G_TYPE_SPAWN_ERROR (g_spawn_error_get_type())
GType g_spawn_flags_get_type (void);
#define G_TYPE_SPAWN_FLAGS (g_spawn_flags_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gstrfuncs.h" */
GType g_ascii_type_get_type (void);
#define G_TYPE_ASCII_TYPE (g_ascii_type_get_type())
GType g_number_parser_error_get_type (void);
#define G_TYPE_NUMBER_PARSER_ERROR (g_number_parser_error_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gtestutils.h" */
GType g_test_trap_flags_get_type (void);
#define G_TYPE_TEST_TRAP_FLAGS (g_test_trap_flags_get_type())
GType g_test_subprocess_flags_get_type (void);
#define G_TYPE_TEST_SUBPROCESS_FLAGS (g_test_subprocess_flags_get_type())
GType g_test_result_get_type (void);
#define G_TYPE_TEST_RESULT (g_test_result_get_type())
GType g_test_log_type_get_type (void);
#define G_TYPE_TEST_LOG_TYPE (g_test_log_type_get_type())
GType g_test_file_type_get_type (void);
#define G_TYPE_TEST_FILE_TYPE (g_test_file_type_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gthread.h" */
GType g_thread_error_get_type (void);
#define G_TYPE_THREAD_ERROR (g_thread_error_get_type())
GType g_once_status_get_type (void);
#define G_TYPE_ONCE_STATUS (g_once_status_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gtimezone.h" */
GType g_time_type_get_type (void);
#define G_TYPE_TIME_TYPE (g_time_type_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/guri.h" */
GType g_uri_flags_get_type (void);
#define G_TYPE_URI_FLAGS (g_uri_flags_get_type())
GType g_uri_hide_flags_get_type (void);
#define G_TYPE_URI_HIDE_FLAGS (g_uri_hide_flags_get_type())
GType g_uri_params_flags_get_type (void);
#define G_TYPE_URI_PARAMS_FLAGS (g_uri_params_flags_get_type())
GType g_uri_error_get_type (void);
#define G_TYPE_URI_ERROR (g_uri_error_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gutils.h" */
GType g_user_directory_get_type (void);
#define G_TYPE_USER_DIRECTORY (g_user_directory_get_type())
GType g_format_size_flags_get_type (void);
#define G_TYPE_FORMAT_SIZE_FLAGS (g_format_size_flags_get_type())

/* enumerations from "/usr/include/glib-2.0/glib/gvariant.h" */
GType g_variant_class_get_type (void);
#define G_TYPE_VARIANT_CLASS (g_variant_class_get_type())
GType g_variant_parse_error_get_type (void);
#define G_TYPE_VARIANT_PARSE_ERROR (g_variant_parse_error_get_type())

/* enumerations from "/usr/include/glib-2.0/gobject/gsignal.h" */
GType g_signal_flags_get_type (void);
#define G_TYPE_SIGNAL_FLAGS (g_signal_flags_get_type())
GType g_connect_flags_get_type (void);
#define G_TYPE_CONNECT_FLAGS (g_connect_flags_get_type())
GType g_signal_match_type_get_type (void);
#define G_TYPE_SIGNAL_MATCH_TYPE (g_signal_match_type_get_type())

G_END_DECLS

#endif /* __GLIB_ENUM_TYPES_H__ */

/* Generated data ends here */

